@@ -218,4 +218,15 @@ extern const char bfs_ldlibs[];
 #  define _target_clones(...)
 #endif
 
+/**
+ * Optimization hint to not unroll a loop.
+ */
+#if BFS_HAS_PRAGMA_NOUNROLL
+#  define _nounroll _Pragma("nounroll")
+#elif __GNUC__ && !__clang__
+#  define _nounroll _Pragma("GCC unroll 0")
+#else
+#  define _nounroll
+#endif
+
 #endif // BFS_H
