@@ -1,9 +1,10 @@
 // Copyright © Tavian Barnes <tavianator@tavianator.com>
 // SPDX-License-Identifier: 0BSD
 
-#include "prelude.h"
 #include "exec.h"
+
 #include "alloc.h"
+#include "bfs.h"
 #include "bfstd.h"
 #include "bftw.h"
 #include "color.h"
@@ -11,6 +12,7 @@
 #include "diag.h"
 #include "dstring.h"
 #include "xspawn.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdarg.h>
