@@ -19,6 +19,8 @@
 
 #if __has_include(<sys/extattr.h>) || __has_include(<sys/xattr.h>)
 #  define BFS_CAN_CHECK_XATTRS true
+#else
+#  define BFS_CAN_CHECK_XATTRS false
 #endif
 
 struct BFTW;
@@ -26,7 +28,7 @@ struct BFTW;
 /**
  * Check if a file has a non-trivial Access Control List.
  *
- * @param ftwbuf
+ * @ftwbuf
  *         The file to check.
  * @return
  *         1 if it does, 0 if it doesn't, or -1 if an error occurred.
@@ -36,7 +38,7 @@ int bfs_check_acl(const struct BFTW *ftwbuf);
 /**
  * Check if a file has a non-trivial capability set.
  *
- * @param ftwbuf
+ * @ftwbuf
  *         The file to check.
  * @return
  *         1 if it does, 0 if it doesn't, or -1 if an error occurred.
@@ -46,7 +48,7 @@ int bfs_check_capabilities(const struct BFTW *ftwbuf);
 /**
  * Check if a file has any extended attributes set.
  *
- * @param ftwbuf
+ * @ftwbuf
  *         The file to check.
  * @return
  *         1 if it does, 0 if it doesn't, or -1 if an error occurred.
@@ -56,9 +58,9 @@ int bfs_check_xattrs(const struct BFTW *ftwbuf);
 /**
  * Check if a file has an extended attribute with the given name.
  *
- * @param ftwbuf
+ * @ftwbuf
  *         The file to check.
- * @param name
+ * @name
  *         The name of the xattr to check.
  * @return
  *         1 if it does, 0 if it doesn't, or -1 if an error occurred.
@@ -68,7 +70,7 @@ int bfs_check_xattr_named(const struct BFTW *ftwbuf, const char *name);
 /**
  * Get a file's SELinux context
  *
- * @param ftwbuf
+ * @ftwbuf
  *         The file to check.
  * @return
  *         The file's SELinux context, or NULL on failure.
