@@ -279,12 +279,11 @@ _noinline
 static uintptr_t ioq_slot_wait(struct ioqq *ioqq, ioq_slot *slot, uintptr_t value) {
 	uintptr_t ret;
 
-	// Try spinning a few times before blocking
+	// Try spinning a few times (with exponential backoff) before blocking
 	_nounroll
-	for (int i = 0; i < 10; ++i) {
-		// Exponential backoff
+	for (int i = 1; i < 1024; i *= 2) {
 		_nounroll
-		for (int j = 0; j < (1 << i); ++j) {
+		for (int j = 0; j < i; ++j) {
 			spin_loop();
 		}
 
@@ -618,7 +617,7 @@ static void ioq_dispatch_sync(struct ioq *ioq, struct ioq_ent *ent) {
 		case IOQ_NOP:
 			if (ent->nop.type == IOQ_NOP_HEAVY) {
 				// A fast, no-op syscall
-				getpid();
+				getppid();
 			}
 			ent->result = 0;
 			return;
@@ -847,7 +846,7 @@ static struct io_uring_sqe *ioq_dispatch_async(struct ioq_ring_state *state, str
 			sqe = ioq_get_sqe(state);
 			struct ioq_stat *args = &ent->stat;
 			int flags = bfs_statx_flags(args->flags);
-			unsigned int mask = STATX_BASIC_STATS | STATX_BTIME;
+			unsigned int mask = bfs_statx_mask();
 			io_uring_prep_statx(sqe, args->dfd, args->path, flags, mask, args->xbuf);
 		}
 #endif
@@ -1037,7 +1036,7 @@ static int ioq_ring_init(struct ioq *ioq, struct ioq_thread *thread) {
 			ioq_ring_probe_flags(&params, IORING_SETUP_SINGLE_ISSUER);
 #  endif
 #  ifdef IORING_SETUP_DEFER_TASKRUN
-			// Don't interrupt us aggresively with completion events
+			// Don't interrupt us aggressively with completion events
 			ioq_ring_probe_flags(&params, IORING_SETUP_DEFER_TASKRUN);
 #  endif
 		}
