@@ -20,13 +20,14 @@
  *     - bftw.[ch]     (an extended version of nftw(3))
  *
  * - Utilities:
+ *     - prelude.h     (feature test macros; automatically included)
  *     - alloc.[ch]    (memory allocation)
  *     - atomic.h      (atomic operations)
  *     - bar.[ch]      (a terminal status bar)
  *     - bit.h         (bit manipulation)
+ *     - bfs.h         (configuration and fundamental utilities)
  *     - bfstd.[ch]    (standard library wrappers/polyfills)
  *     - color.[ch]    (for pretty terminal colors)
- *     - prelude.h     (configuration and feature/platform detection)
  *     - diag.[ch]     (formats diagnostic messages)
  *     - dir.[ch]      (a directory API facade)
  *     - dstring.[ch]  (a dynamic string library)
@@ -47,12 +48,12 @@
  *     - xtime.[ch]    (date/time handling utilities)
  */
 
-#include "prelude.h"
 #include "bfstd.h"
 #include "ctx.h"
 #include "diag.h"
 #include "eval.h"
 #include "parse.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <locale.h>
