@@ -1,4 +1,4 @@
 stderr=$(invoke_bfs -warn -O9 basic 2>&1 >"$OUT")
-[ -n "$stderr" ] || return 1
+[ -n "$stderr" ]
 sort_output
 diff_output
