@@ -1,11 +1,11 @@
 // Copyright © Tavian Barnes <tavianator@tavianator.com>
 // SPDX-License-Identifier: 0BSD
 
-#include "prelude.h"
 #include "tests.h"
+
 #include "bfstd.h"
 #include "diag.h"
-#include <errno.h>
+
 #include <langinfo.h>
 #include <stdlib.h>
 #include <string.h>
