@@ -5,11 +5,10 @@
  * Entry point for unit tests.
  */
 
-#include "prelude.h"
 #include "tests.h"
-#include "bfstd.h"
+
 #include "color.h"
-#include "thread.h"
+
 #include <locale.h>
 #include <stdio.h>
 #include <stdlib.h>
