@@ -1,11 +1,12 @@
 // Copyright © Tavian Barnes <tavianator@tavianator.com>
 // SPDX-License-Identifier: 0BSD
 
-#include "prelude.h"
 #include "tests.h"
-#include "xtime.h"
-#include "bfstd.h"
+
+#include "bfs.h"
 #include "diag.h"
+#include "xtime.h"
+
 #include <errno.h>
 #include <limits.h>
 #include <stdint.h>
